/**
 * \file
 *
 * \brief irmp test
 *
 */

/**
 * \mainpage irmp test doxygen documentation
 *
 * \par irmp test
 *
 * Bare minimum empty user application template
 *
 * \par Content
 *
 * -# Include the ASF header files (through asf.h)
 * -# Minimal main function that starts with a call to board_init()
 * -# "Insert application code here" comment
 *
 */

#define USE_SEND_BACK

/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
#include <asf.h>
#include <irmp.h>

#define RC_TIMER       &TCC0
#define RC_TIMER_RES   4000000UL

#ifdef USE_SEND_BACK
#include <irsnd.c>
#endif


IRMP_DATA irmp_data;

static void rc_isr_handler(void)
{
  static uint16_t isr_count = 0;
#ifdef USE_SEND_BACK
  if (! irsnd_ISR()) {
    // if not sending
    ioport_set_pin_level(LED2, IOPORT_PIN_LEVEL_HIGH);
    irmp_ISR();
  }
#else
  irmp_ISR();
#endif
  if (isr_count++ > F_INTERRUPTS) {
    isr_count = 0;
    // blinke im sekunden tackt
    ioport_toggle_pin_level(LED3);
  }
}

static void rc_init(void)
{
#ifdef USE_SEND_BACK
  irsnd_init();
#endif
  irmp_init();
  irq_initialize_vectors(); // done by pmic_init()
  cpu_irq_enable();

  tc_enable(RC_TIMER);
  // Enable overflow interrupt
  tc_set_overflow_interrupt_level(RC_TIMER, TC_INT_LVL_LO);
  // Configure TC in normal mode
  tc_set_wgm(RC_TIMER, TC_WG_NORMAL);
  // Configure call back interrupt
  tc_set_overflow_interrupt_callback(RC_TIMER, rc_isr_handler);
  // Configure TC period and and resolution
  tc_set_resolution(RC_TIMER, RC_TIMER_RES);
  uint32_t reso = tc_get_resolution(RC_TIMER);
  if (reso != RC_TIMER_RES) {
    printf("warning timer resolution not same actual=%ld chosen=%ld period=%ld",
      reso, RC_TIMER_RES, (reso / F_INTERRUPTS));
  }
  tc_write_period(RC_TIMER, (reso / F_INTERRUPTS));
}

static void show_rc_data(void)
{
  static uint16_t rc_repeat_count = 0;
  if (irmp_data.flags & IRMP_FLAG_REPETITION) {
    // user immer noch auf gleicher taste
    ioport_toggle_pin_level(LED0);
    if (rc_repeat_count++ > 3) {
//      printf("R");
      ioport_toggle_pin_level(LED1);
    }
  }
  else {
    // erster tastendruck
    ioport_set_pin_level(LED0, IOPORT_PIN_LEVEL_LOW);
    ioport_set_pin_level(LED1, IOPORT_PIN_LEVEL_HIGH);
    rc_repeat_count = 0;
#ifdef USE_SEND_BACK
    irmp_data.flags = 0;    // reset flags!
    ioport_set_pin_level(LED2, IOPORT_PIN_LEVEL_LOW);
    irsnd_send_data(&irmp_data, 0);
#endif
    printf("\n%d", irmp_data.protocol);
#if (IRMP_PROTOCOL_NAMES  == 1)       // 1: access protocol names, 0: do not (default),
    printf(" %s", irmp_protocol_names[irmp_data.protocol]);
#endif
    uint16_t offset = 0;
    if (irmp_data.protocol == IRMP_RC6A_PROTOCOL) {
      offset = 1024;
    }
    printf(" adr=%d cmd=%d ", irmp_data.address, (irmp_data.command - offset));
  }
}

int main(void)
{
  const usart_serial_options_t usart_serial_options = {
    .baudrate     = USART_SERIAL_BAUDRATE,
    .charlength   = USART_SERIAL_CHAR_LENGTH,
    .paritytype   = USART_SERIAL_PARITY,
    .stopbits     = USART_SERIAL_STOP_BIT
  };
  
  pmic_init();
 	board_init();
  sysclk_init();
  stdio_serial_init(USART_SERIAL, &usart_serial_options);
  sleepmgr_init();
	rc_init();

  do {
    if (irmp_get_data(&irmp_data)) {
      show_rc_data();
    }
    // Sleep until interrupt triggers.
    sleepmgr_enter_sleep();
  } while (1);
}
